import { Op } from 'sequelize';
import { VipModel, VipLevel, VipCategory } from './models/index.ts';
import { app } from '@/app.ts';
import './list.ts';

app
  .route({
    path: 'vip',
    key: 'list',
    middleware: ['auth-admin'],
  })
  .define(async (ctx) => {
    const tokenUser = ctx.state.tokenUser;
    const { page = 1, pageSize = 20, search, sort = 'DESC', level, category, userId } = ctx.query;

    const whereCondition: any = {};
    if (userId) {
      whereCondition.userId = userId;
    }

    if (level) {
      whereCondition.level = level;
    }

    if (category) {
      whereCondition.category = category;
    }

    if (search) {
      whereCondition[Op.or] = [{ title: { [Op.like]: `%${search}%` } }];
    }

    const { rows: vips, count } = await VipModel.findAndCountAll({
      where: whereCondition,
      offset: (page - 1) * pageSize,
      limit: pageSize,
      order: [['updatedAt', sort]],
    });

    ctx.body = {
      list: vips,
      pagination: {
        page,
        current: page,
        pageSize,
        total: count,
      },
      levels: Object.values(VipLevel),
      categories: Object.values(VipCategory),
    };
  })
  .addTo(app);

app
  .route({
    path: 'vip',
    key: 'update',
    middleware: ['auth-admin'],
  })
  .define(async (ctx) => {
    const tokenUser = ctx.state.tokenUser;
    const { id, data, updatedAt: _clear, createdAt: _clear2, ...rest } = ctx.query.data;
    let vip: VipModel;
    let isNew = false;

    if (id) {
      vip = await VipModel.findByPk(id);
      // if (!vip || vip.userId !== tokenUser.uid) {
      //   ctx.throw(403, 'No permission');
      // }
    } else {
      vip = await VipModel.create({
        data: data || {},
        ...rest,
        // userId: tokenUser.uid,
      });
      isNew = true;
    }

    if (!vip) {
      ctx.throw(404, 'VIP record not found');
    }

    if (!isNew) {
      vip = await vip.update({
        data: { ...vip.data, ...(data || {}) },
        ...rest,
      });
    }

    ctx.body = vip;
  })
  .addTo(app);

app
  .route({
    path: 'vip',
    key: 'delete',
    middleware: ['auth-admin'],
  })
  .define(async (ctx) => {
    const { id, force = false } = ctx.query.data || {};
    if (!id) {
      ctx.throw(400, 'id is required');
    }
    const vip = await VipModel.findByPk(id);
    await vip.destroy({ force });
    ctx.body = vip;
  })
  .addTo(app);

app
  .route({
    path: 'vip',
    key: 'get',
    middleware: ['auth-admin'],
  })
  .define(async (ctx) => {
    const tokenUser = ctx.state.tokenUser;
    const { id } = ctx.query.data || {};

    if (!id) {
      ctx.throw(400, 'id is required');
    }

    const vip = await VipModel.findByPk(id);

    if (!vip || vip.userId !== tokenUser.uid) {
      ctx.throw(403, 'No permission');
    }

    ctx.body = vip;
  })
  .addTo(app);

// 获取VIP等级和类别信息
app
  .route({
    path: 'vip',
    key: 'config',
    middleware: ['auth-admin'],
  })
  .define(async (ctx) => {
    ctx.body = {
      levels: Object.values(VipLevel),
      categories: Object.values(VipCategory),
    };
  })
  .addTo(app);
